==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/base/treeview.h <==
/*
 * Copyright (C) 2005-2008, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TREEVIEW_H
# define TREEVIEW_H

# include <string>
# include <vector>

# include "treegrouproot.h"
# include "links.h"
# include "perm.h"

class TreeKwHandler;

class TreeView
{
  friend class TreeKwHandler;
public:
  
  /**
   * Load errors: these codes may be or'ed together!
   */
  enum eLoadError { eLE_NO_ERROR=0, eLE_LINK=1, eLE_BASEPATH=2, eLE_XML=4, eLE_NOFILE=8, eLE_VERSION=16 };
  
  /**
   * 
   */
  enum eThumbDir { eSrc=0, eHome=1, eSelect=2 };
  
  
public:
  
  /** */
  TreeView(TreeView* mysource);
  
  /** */
  ~TreeView();
  
  /**
   * Is there anything except default data?
   */
  bool hasData();
  
  /**
   * Makes sure parents are expanded so <code>item</code> can be seen.
   * Returns true if one or more parents had to be expanded.
   */
  bool makeVisible(TreeItem* item);
  
  /** 
   * Loads data from a file into @ref root. @ref projectfilePath gets updated.
   * If @ref root already contained data, it will be deleted.
   * Return codes: see eLE...
   */
  int loadFromFile(std::string newProjectfilePath);
  
  /**
   * Save contents of project file
   */
  SimpleXMLTransfer* saveToSXT(std::string newProjectfilePath, bool fConvLinks, int nVersion);
  
  /**
   * If I am a view of some other data, just call 
   * <code>source->saveToProjectFile()</code>. Otherwise, do the work.
   */
  int saveToProjectFile();
  
  /**
   * Save myself as a new project. If I am a view of some other data,
   * I have to transform to real data before! Doing this will not be easy
   * if there already are views of myself. Issue an error in this case (return -1)
   * and don't do anything.
   */
  int saveAsNewProject(std::string newProjectfilePath);
  
  /**
   * 
   */
  TreeItem* resolveInternalLink(std::string linktext);
  
  /**
   * 
   */
  std::string resolveExternalLink(std::string linktext,
                                  TreeItem*   item);
  
  /**
   * Implementation without recursive function calls.
   */
  TreeItem* browserItemPrev(TreeItem* p, bool fShowPics);
  
  /**
   * Implementation without recursive function calls.
   */
  TreeItem* browserItemNext(TreeItem* p, bool fShowPics);
  
  /**
   * toggle mark of an item
   */
  void toggleMarkItem(TreeItem* p);
  
  /**
   * set mark of an item
   */
  void setMarkItem(TreeItem* p);
  
  /**
   * Marks all items in group (non-recursively) which match search criteria sd.
   */
  void markItems(TreeGroupBase* group, SearchData& sd);
  
  /**
   * Reset mark of an item. If p == 0, all items are unmarked.
   */
  void resetMarkItem(TreeItem* p);
  
  /**
   * returns the number of marked items
   */
  unsigned int countMarkedItems();
  
  /**
   * Returns type of marked items
   */
  TreeItem::eItemType markedItemsType();
  
  /**
   * 
   */
  std::string getPicPath(TreeItem* item, int nSize=0);
  
  
  /**
   * is called from LinkInfo to tell that uID (with type type) is missing.
   */
  void itemMissing(unsigned int uID, TreeItem::eItemType type);
  
  /**
   * is called from LinkInfo to tell that uID is not type (as expected).
   */
  void itemWrongType(unsigned int uID, TreeItem::eItemType type);
  
  /**
   * Returns a pointer to the LinkInfo to be used in this TreeView.
   */
  LinkInfo* getLinkInfoList();
  
  /**
   * Returns the highest ID in use.
   */
  unsigned int getIDMax();
  
  /**
   * Adds a TreeItem to the list of linkable items.
   */
  void newLinkableItem(TreeItem* item);
  
  /**
   * 
   */
  void showLinkableItems();
  
  /**
   * 
   */
  std::string getProjectfilePath();
  
  /**
   * 
   */
  int addPicsFromDir(const char*                  picdir, 
                     TreeGroupBase*               lastPopupSingleItem,
                     TreePicData::eDateTimeSource eDTS);
  
  /**
   * 
   */
  int addFile(const char*                  picdir, 
              TreeGroupBase*               lastPopupSingleItem,
              TreePicData::eDateTimeSource eDTS);
  
  /**
   * 
   */
  int updateBasepath(const char* picdir);
  
  
  /**
   * 
   */
  void linkInto_marked(TreeGroupBase* destgroup);
  
  /**
   * 
   */
  void moveInto_marked(TreeGroupBase* destgroup);
  
  /**
   * 
   */
  void moveAfter_marked(TreeItem* destitem);
  
  
  /**
   * Marked items are appended to the list. Usually items are only marked in one group
   * and therefore this method is not recursive.
   */
  void getMarkedItems(std::vector<TreeItem*>& list);
  
  /**
   * 
   */
  std::string getBasepath();
  
  /**
   * See @ref fAddDelPicAllowed
   */
  bool isAddDelAllowed();
  
  /**
   * maybe or'ed together:
   *  0: everything ok
   *  0x01: Group not empty, not deleted.
   *  0x02: This item is destination of a link, not deleted.
   *  0x08: There is a dangling link somewhere. Can be ignored in this case.
   *  0x10: Link count underflow! Critical internal error!
   *  0x20: Link should have been in link info, but could not be found. Critical internal error!
   */
  int deleteItem(TreeItem* item, bool fDelFiles);
  
  /**
   * 
   */
  bool dataSaved();
  
  /**
   * 
   */
  TreeItem* getFirstMarkedItem();
  
  /**
   * Creates a new view of the data below group. Before calling this,
   * make sure that nothing is marked!
   */
  void newViewBySearch(TreeGroupBase* group, SearchData sd);
  
  /**
   * Creates a new view of the data below group. Before calling this,
   * make sure that nothing is marked!
   */
  void newViewWithoutText(TreeGroupBase* group);
  
  /**
   * Returns a new view of my data.
   */
  void newViewByDate(TreeGroupBase* group);
  
  /**
   * 
   */
  TreeView* getSource();
  
  /**
   * 
   */
  StringSet* getHiddenKeywords();
  
  
  /**
   * To exchange data with album settings dialog.
   */
  void exchangeThumbSettings(eThumbDir& th, std::string& tds, bool fWrite);

  /**
   * Expands a list recursively to a set (no duplicates).
   */
  static void RecursiveList(const std::vector<TreeItem*>& list, std::set<TreeItem*>& set);
  
  /**
   * returns path of most recent addition
   */
  std::string GetAddPath() { return(strAddPath); };
  
public:
  
  /** my data  */
  TreeGroupRoot* root;
  
  /** for keyword handling */
  TreeKwHandler* kw;
  
  /** available thumbnail sizes */
  static const unsigned int uThumbSizes[];

  T_Perm* perm;
  
private:
  
  /** The first path is special. */
  std::vector<std::string> basePaths;
  
  /**
   * There are marked items only in this group. Points
   * to 0 if nothing is marked.
   */
  TreeGroupBase* groupWithMarkedItems;
  
  /**
   * Number of marked items in group @ref groupWithMarkedItems
   */
  unsigned int uMarkedItems;
  
  /** the basepath in use */
  std::string basePath;
  
  /**
   * Depends on basepath in use.
   * Does not refer to add and del only, but also to actions on the pictures 
   * themselves, like rotating and editing.
   */
  bool fAddDelPicAllowed;
  
  /** Make sure @ref root is empty. */
  void deleteRoot();
  
  /**
   * Make sure @ref root exists and has a title.
   */
  void createRoot();   
  
  /**
   * If I am a view of some other data, this pointer
   * shows the source. In this case, one should not change 
   * my structure (move items around, delete items, add items). 
   * It is possible
   * to edit properties of a single item (which will usually edit 
   * those properties in the source).
   */
  TreeView* source;
  
  /** 
   * Views created by me. I have to stay alive as long as there
   * are some of them, as they might link back to my data.
   */
  std::vector<TreeView*> views;
  
  /**
   * path to project file
   */
  std::string projectfilePath;
  
  /**
   * Table of linkable items. In some cases this list must not
   * be used, see @ref getLinkInfoList.
   */
  LinkInfo* linkInfo;
  
  /**
   * Strings in this set cause a picture to be hidden when found in keywords.
   */
  StringSet HiddenKeywords;
  
  /**
   * Where to put thumbnails? This should not be read directly,
   * as it depends on @ref source, too. Use @ref getPicPath in any case.
   */
  eThumbDir thumbdir;
  
  /**
   * Valid in case of thumbdir == eSelect. This should not be read directly,
   * as it depends on @ref source, too. Use @ref getPicPath in any case.
   */
  std::string thumbdirselect;
  
  std::string strBasepathRelOriginal;
  
  std::string strAddPath;
};

#endif

==> fotoalbum-0.104.tar.bz2: fotoalbum-0.104/src/base/treeview.h <==
/*
 * Copyright (C) 2005-2008, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TREEVIEW_H
# define TREEVIEW_H

# include <string>
# include <vector>

# include "treegrouproot.h"
# include "links.h"
# include "perm.h"

class TreeKwHandler;

class TreeView
{
  friend class TreeKwHandler;
public:
  
  /**
   * Load errors: these codes may be or'ed together!
   */
  enum eLoadError { eLE_NO_ERROR=0, eLE_LINK=1, eLE_BASEPATH=2, eLE_XML=4, eLE_NOFILE=8, eLE_VERSION=16, eLE_NotUTF8=32, eLE_UTFConv=64 };
  
  /**
   * 
   */
  enum eThumbDir { eSrc=0, eHome=1, eSelect=2 };
  
  
public:
  
  /** */
  TreeView(TreeView* mysource);
  
  /** */
  ~TreeView();
  
  /**
   * Is there anything except default data?
   */
  bool hasData();
  
  /**
   * Makes sure parents are expanded so <code>item</code> can be seen.
   * Returns true if one or more parents had to be expanded.
   */
  bool makeVisible(TreeItem* item);
  
  /** 
   * Loads data from a file into @ref root. @ref projectfilePath gets updated.
   * If @ref root already contained data, it will be deleted.
   * Return codes: see eLE...
   */
  int loadFromFile(std::string newProjectfilePath, std::string strCharset);
  
  /**
   * Save contents of project file
   */
  SimpleXMLTransfer* saveToSXT(std::string newProjectfilePath, bool fConvLinks, int nVersion);
  
  /**
   * If I am a view of some other data, just call 
   * <code>source->saveToProjectFile()</code>. Otherwise, do the work.
   */
  int saveToProjectFile();
  
  /**
   * Save myself as a new project. If I am a view of some other data,
   * I have to transform to real data before! Doing this will not be easy
   * if there already are views of myself. Issue an error in this case (return -1)
   * and don't do anything.
   */
  int saveAsNewProject(std::string newProjectfilePath);
  
  /**
   * 
   */
  TreeItem* resolveInternalLink(std::string linktext);
  
  /**
   * 
   */
  std::string resolveExternalLink(std::string linktext,
                                  TreeItem*   item);
  
  /**
   * Implementation without recursive function calls.
   */
  TreeItem* browserItemPrev(TreeItem* p, bool fShowPics);
  
  /**
   * Implementation without recursive function calls.
   */
  TreeItem* browserItemNext(TreeItem* p, bool fShowPics);
  
  /**
   * toggle mark of an item
   */
  void toggleMarkItem(TreeItem* p);
  
  /**
   * set mark of an item
   */
  void setMarkItem(TreeItem* p);
  
  /**
   * Marks all items in group (non-recursively) which match search criteria sd.
   */
  void markItems(TreeGroupBase* group, SearchData& sd);
  
  /**
   * Reset mark of an item. If p == 0, all items are unmarked.
   */
  void resetMarkItem(TreeItem* p);
  
  /**
   * returns the number of marked items
   */
  unsigned int countMarkedItems();
  
  /**
   * Returns type of marked items
   */
  TreeItem::eItemType markedItemsType();
  
  /**
   * 
   */
  std::string getPicPath(TreeItem* item, int nSize=0);
  
  
  /**
   * is called from LinkInfo to tell that uID (with type type) is missing.
   */
  void itemMissing(unsigned int uID, TreeItem::eItemType type);
  
  /**
   * is called from LinkInfo to tell that uID is not type (as expected).
   */
  void itemWrongType(unsigned int uID, TreeItem::eItemType type);
  
  /**
   * Returns a pointer to the LinkInfo to be used in this TreeView.
   */
  LinkInfo* getLinkInfoList();
  
  /**
   * Returns the highest ID in use.
   */
  unsigned int getIDMax();
  
  /**
   * Adds a TreeItem to the list of linkable items.
   */
  void newLinkableItem(TreeItem* item);
  
  /**
   * 
   */
  void showLinkableItems();
  
  /**
   * 
   */
  std::string getProjectfilePath();
  
  /**
   * 
   */
  int addPicsFromDir(const char*                  picdir, 
                     TreeGroupBase*               lastPopupSingleItem,
                     TreePicData::eDateTimeSource eDTS);
  
  /**
   * 
   */
  int addFile(const char*                  picdir, 
              TreeGroupBase*               lastPopupSingleItem,
              TreePicData::eDateTimeSource eDTS);
  
  /**
   * 
   */
  int updateBasepath(const char* picdir);
  
  
  /**
   * 
   */
  void linkInto_marked(TreeGroupBase* destgroup);
  
  /**
   * 
   */
  void moveInto_marked(TreeGroupBase* destgroup);
  
  /**
   * 
   */
  void moveAfter_marked(TreeItem* destitem);
  
  
  /**
   * Marked items are appended to the list. Usually items are only marked in one group
   * and therefore this method is not recursive.
   */
  void getMarkedItems(std::vector<TreeItem*>& list);
  
  /**
   * 
   */
  std::string getBasepath();
  
  /**
   * See @ref fAddDelPicAllowed
   */
  bool isAddDelAllowed();
  
  /**
   * maybe or'ed together:
   *  0: everything ok
   *  0x01: Group not empty, not deleted.
   *  0x02: This item is destination of a link, not deleted.
   *  0x08: There is a dangling link somewhere. Can be ignored in this case.
   *  0x10: Link count underflow! Critical internal error!
   *  0x20: Link should have been in link info, but could not be found. Critical internal error!
   */
  int deleteItem(TreeItem* item, bool fDelFiles);
  
  /**
   * 
   */
  bool dataSaved();
  
  /**
   * 
   */
  TreeItem* getFirstMarkedItem();
  
  /**
   * Creates a new view of the data below group. Before calling this,
   * make sure that nothing is marked!
   */
  void newViewBySearch(TreeGroupBase* group, SearchData sd);
  
  /**
   * Creates a new view of the data below group. Before calling this,
   * make sure that nothing is marked!
   */
  void newViewWithoutText(TreeGroupBase* group);
  
  /**
   * Returns a new view of my data.
   */
  void newViewByDate(TreeGroupBase* group);
  
  /**
   * 
   */
  TreeView* getSource();
  
  /**
   * 
   */
  StringSet* getHiddenKeywords();
  
  
  /**
   * To exchange data with album settings dialog.
   */
  void exchangeThumbSettings(eThumbDir& th, std::string& tds, bool fWrite);

  /**
   * Expands a list recursively to a set (no duplicates).
   */
  static void RecursiveList(const std::vector<TreeItem*>& list, std::set<TreeItem*>& set);
  
  /**
   * returns path of most recent addition
   */
  std::string GetAddPath() { return(strAddPath); };
  
public:
  
  /** my data  */
  TreeGroupRoot* root;
  
  /** for keyword handling */
  TreeKwHandler* kw;
  
  /** available thumbnail sizes */
  static const unsigned int uThumbSizes[];

  T_Perm* perm;
  
private:
  
  /** The first path is special. */
  std::vector<std::string> basePaths;
  
  /**
   * There are marked items only in this group. Points
   * to 0 if nothing is marked.
   */
  TreeGroupBase* groupWithMarkedItems;
  
  /**
   * Number of marked items in group @ref groupWithMarkedItems
   */
  unsigned int uMarkedItems;
  
  /** the basepath in use */
  std::string basePath;
  
  /**
   * Depends on basepath in use.
   * Does not refer to add and del only, but also to actions on the pictures 
   * themselves, like rotating and editing.
   */
  bool fAddDelPicAllowed;
  
  /** Make sure @ref root is empty. */
  void deleteRoot();
  
  /**
   * Make sure @ref root exists and has a title.
   */
  void createRoot();   
  
  /**
   * If I am a view of some other data, this pointer
   * shows the source. In this case, one should not change 
   * my structure (move items around, delete items, add items). 
   * It is possible
   * to edit properties of a single item (which will usually edit 
   * those properties in the source).
   */
  TreeView* source;
  
  /** 
   * Views created by me. I have to stay alive as long as there
   * are some of them, as they might link back to my data.
   */
  std::vector<TreeView*> views;
  
  /**
   * path to project file
   */
  std::string projectfilePath;
  
  /**
   * Table of linkable items. In some cases this list must not
   * be used, see @ref getLinkInfoList.
   */
  LinkInfo* linkInfo;
  
  /**
   * Strings in this set cause a picture to be hidden when found in keywords.
   */
  StringSet HiddenKeywords;
  
  /**
   * Where to put thumbnails? This should not be read directly,
   * as it depends on @ref source, too. Use @ref getPicPath in any case.
   */
  eThumbDir thumbdir;
  
  /**
   * Valid in case of thumbdir == eSelect. This should not be read directly,
   * as it depends on @ref source, too. Use @ref getPicPath in any case.
   */
  std::string thumbdirselect;
  
  std::string strBasepathRelOriginal;
  
  std::string strAddPath;
};

#endif
